@@ -47,6 +47,15 @@ from .testutils import subprocess_pickle_echo
 from .testutils import assert_run_python_script
 
 
+class RaiserOnPickle(object):
+
+    def __init__(self, exc):
+        self.exc = exc
+
+    def __reduce__(self):
+        raise self.exc
+
+
 def pickle_depickle(obj, protocol=cloudpickle.DEFAULT_PROTOCOL):
     """Helper function to test whether object pickled with cloudpickle can be
     depickled with pickle
@@ -213,7 +222,7 @@ class CloudPickleTest(unittest.TestCase):
 
     def test_unhashable_closure(self):
         def f():
-            s = set((1, 2))  # mutable set is unhashable
+            s = {1, 2}  # mutable set is unhashable
 
             def g():
                 return len(s)
@@ -338,15 +347,6 @@ class CloudPickleTest(unittest.TestCase):
         else:  # skip if scipy is not available
             pass
 
-    def test_save_unsupported(self):
-        sio = StringIO()
-        pickler = cloudpickle.CloudPickler(sio, 2)
-
-        with pytest.raises(pickle.PicklingError) as excinfo:
-            pickler.save_unsupported("test")
-
-        assert "Cannot pickle objects of type" in str(excinfo.value)
-
     def test_loads_namespace(self):
         obj = 1, 2, 3, 4
         returned_obj = cloudpickle.loads(cloudpickle.dumps(obj))
@@ -494,7 +494,7 @@ class CloudPickleTest(unittest.TestCase):
         nvars = 65537 + 258
         names = ['g%d' % i for i in range(1, nvars)]
         r = random.Random(42)
-        d = dict([(name, r.randrange(100)) for name in names])
+        d = {name: r.randrange(100) for name in names}
         # def f(x):
         #     x = g1, g2, ...
         #     return zlib.crc32(bytes(bytearray(x)))
@@ -613,8 +613,8 @@ class CloudPickleTest(unittest.TestCase):
             msg='cell contents not set correctly',
         )
 
-    def test_logger(self):
-        logger = logging.getLogger('cloudpickle.dummy_test_logger')
+    def check_logger(self, name):
+        logger = logging.getLogger(name)
         pickled = pickle_depickle(logger, protocol=self.protocol)
         self.assertTrue(pickled is logger, (pickled, logger))
 
@@ -633,7 +633,13 @@ class CloudPickleTest(unittest.TestCase):
         out, _ = proc.communicate()
         self.assertEqual(proc.wait(), 0)
         self.assertEqual(out.strip().decode(),
-                         'INFO:cloudpickle.dummy_test_logger:hello')
+                         'INFO:{}:hello'.format(logger.name))
+
+    def test_logger(self):
+        # logging.RootLogger object
+        self.check_logger(None)
+        # logging.Logger object
+        self.check_logger('cloudpickle.dummy_test_logger')
 
     def test_abc(self):
 
@@ -687,7 +693,7 @@ class CloudPickleTest(unittest.TestCase):
         self.assertEqual(depickled3.x, 3)
         self.assertEqual(len(weakset), 2)
 
-        self.assertEqual(set(weakset), set([depickled1, depickled2]))
+        self.assertEqual(set(weakset), {depickled1, depickled2})
 
     def test_faulty_module(self):
         for module_name in ['_faulty_module', '_missing_module', None]:
@@ -862,6 +868,32 @@ class CloudPickleTest(unittest.TestCase):
             b'\x14NtR.')
         self.assertEquals(42, cloudpickle.loads(pickled)(42))
 
+    def test_pickle_reraise(self):
+        for exc_type in [Exception, ValueError, TypeError, RuntimeError]:
+            obj = RaiserOnPickle(exc_type("foo"))
+            with pytest.raises((exc_type, pickle.PicklingError)):
+                cloudpickle.dumps(obj)
+
+    def test_unhashable_function(self):
+        d = {'a': 1}
+        depickled_method = pickle_depickle(d.get)
+        self.assertEquals(depickled_method('a'), 1)
+        self.assertEquals(depickled_method('b'), None)
+
+    def test_itertools_count(self):
+        counter = itertools.count(1, step=2)
+
+        # advance the counter a bit
+        next(counter)
+        next(counter)
+
+        new_counter = pickle_depickle(counter, protocol=self.protocol)
+
+        self.assertTrue(counter is not new_counter)
+
+        for _ in range(10):
+            self.assertEqual(next(counter), next(new_counter))
+
 
 class Protocol2CloudPickleTest(CloudPickleTest):
 
